if (typeof(PhusionPassenger) !== 'undefined') {
    PhusionPassenger.configure({ autoInstall: false });
}

const express = require('express');
const cors = require('cors')
const http = require('http');
const WebSocket = require('ws');
const jwt = require('jsonwebtoken');
const net = require('net');
const zlib = require('zlib');
const userRouter = require('./routes/users'); // Adjust the path as necessary

const SERVER_PORT = 49152;
const WS_PORT = 12345;    // Port for WebSocket server (sending to React app)
const HTTP_PORT = process.env.PORT || 3000; // Ensure this does not conflict with your other server ports




// Initialize Express app and routes
const app = express();
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cors())


function normalizePort(val) {
var port = parseInt(val, 10);

if (isNaN(port)) {
    // named pipe
    return val;
}

if (port >= 0) {
    // port number
    return port;
}

return false;
}
var port = normalizePort(process.env.PORT || '3000');
app.set('port', port);

app.use('/user',userRouter)
app.get('/test',(req,res)=>{
    res.send("Working")
})


const wss = new WebSocket.Server({ port: WS_PORT });

// console.log(`WebSocket server started on port ${WS_PORT}`);

wss.on('connection', (ws, req) => {
    const token = new URLSearchParams(req.url.split('?')[1]).get('token');
    try {
        const decoded = jwt.verify(token, 'croxsol');
        ws.clientId = decoded.id; // Attach user ID to the WebSocket object
        ws.role = decoded.role;
    } catch (error) {
        ws.close(); // Close connection if token is invalid
    }
});

const broadcast = (clientId,message) => {
    console.log("Clients",wss.clients.size);
    wss.clients.forEach((client) => {
        console.log(client.clientId,clientId);
        if (client.readyState === WebSocket.OPEN && (client.clientId == clientId || client.role == 'admin' ) ) {
            console.log(clientId + " sending message to " + client.clientId);
            client.send(JSON.stringify(message));
        }
    });
};


const tcpServer = net.createServer();

tcpServer.on('connection', (socket) => {
    console.log(`Client connected: ${socket.remoteAddress}:${socket.remotePort}`);
    let receivedBuffer = Buffer.alloc(0);

    socket.on('data', (chunk) => {
        receivedBuffer = Buffer.concat([receivedBuffer, chunk]);
        processReceivedData(socket, receivedBuffer)
            .then(newBuffer => receivedBuffer = newBuffer)
            .catch(err => console.error('Error processing data:', err));
    });

    socket.on('close', () => {
        console.log(`Connection closed: ${socket.remoteAddress}:${socket.remotePort}`);
    });

    socket.on('error', (err) => {
        console.error('Socket error:', err);
    });
});

// Tcp Server listen
tcpServer.listen(SERVER_PORT, () => {
    console.log(`Server listening on port ${SERVER_PORT}`);
});


const processReceivedData = async (socket, buffer) => {
    while (buffer.length >= 4) {
        const totalSize = buffer.readUInt32BE(0);
        if (buffer.length - 4 >= totalSize) {
            const data = buffer.slice(4, 4 + totalSize);
            buffer = buffer.slice(4 + totalSize);
            // await processData(socket, data);

            const firstSeparatorIndex = data.indexOf('|'.charCodeAt(0));
            const clientId = data.slice(0, firstSeparatorIndex).toString();

            const remainingData = data.slice(firstSeparatorIndex + 1);
            const secondSeparatorIndex = remainingData.indexOf('|'.charCodeAt(0));
            const pcName = remainingData.slice(0, secondSeparatorIndex).toString();

            const imageData = remainingData.slice(secondSeparatorIndex + 1);

            console.log(`Data received from ${clientId} ${pcName}`);

            zlib.unzip(imageData, (err, buffer) => {
                if (err) {
                    console.error(`Decompression error from ${clientId}:`, err);
                    return;
                }

                
                // Broadcast the image to all connected WebSocket clients
                const base64Image = buffer.toString('base64');
                broadcast(clientId,{pcName, image: base64Image });          
                
                // const filename = `${clientId}_${Date.now()}.jpg`;
                // fs.writeFile(filename, buffer, (err) => {
                //     if (err) {
                //         console.error(`File writing error for ${clientId}:`, err);
                //         return;
                //     }
                //     console.log(`Image from ${clientId} saved as ${filename}`);
                // });
            });
        } else {
            break;
        }
    }
    return buffer;
};

if (typeof(PhusionPassenger) !== 'undefined') {
    app.listen('passenger');
} else {
    app.listen(HTTP_PORT, () => {
        console.log(`HTTP server listening on port ${HTTP_PORT}`);
    });
}



